﻿using System;
using System.Collections.Generic;
using System.Net;
using System.Net.Http;
using System.Web.Http;
using Kreta.BusinessLogic.Security;
using Kreta.Client.KGR;
using Kreta.Client.KGR.Request;
using Kreta.Core.KozpontiModels.TargyiEszkozModels;
using Kreta.Resources;
using Kreta.Web.Helpers.Error;
using Kreta.Web.Helpers.Grid;
using Kreta.Web.Security;

namespace Kreta.Web.Areas.GRModul.ApiControllers
{
    [ApiRoleClaimsAuthorize(true)]
    [ApiRolePackageAuthorize(KretaClaimPackages.Adminisztrator.ClaimValue)]
    public class TargyiEszkozokApiController : ApiController
    {
        private readonly IKGRClient kgrClient;

        public TargyiEszkozokApiController(IKGRClient kgrClient)
        {
            this.kgrClient = kgrClient ?? throw new ArgumentNullException(nameof(kgrClient));
        }

        public HttpResponseMessage Get()
        {
            var kozpontiResult = kgrClient.KGREszkozList(new EszkozHeaderRequest() { IntezmenyGuid = ClaimData.IntezmenyGuid.ToString(), IntezmenyAzonosito = ClaimData.IntezmenyAzonosito });

            List<TargyiEszkozResponseModel> result = new List<TargyiEszkozResponseModel>();
            if (kozpontiResult != null)
            {
                var cnt = 0;
                foreach (var item in kozpontiResult)
                {
                    result.Add(new TargyiEszkozResponseModel()
                    {
                        EszkozLeltarszam = item.Leltarszam,
                        EszkozPenzugyiKozpontNev = "",
                        EszkozLeltarkorzetNev = "",
                        EszkozMennyisegNev = "",
                        EszkozMennyiseg = 1,
                        EszkozMuszakicsoportkodNev = "",
                        EszkozMegnevezes3 = "",
                        EszkozMegnevezes2 = "",
                        EszkozGyariSzam = item.GyariSzam,
                        EszkozMegnevezes = item.Megnevezes,
                        SzuletesiHely = "",
                        AdoAzonosito = "",
                        OktatasiAzonosito = "",
                        AlkalmazottNev = item.AlkalmazottNev,
                        AlkalmazottId = item.AlkalmazottId,
                        SzTSzAzonosito = item.SztszAzonosito,
                        FunkcioTerulet = "",
                        SorId = cnt,
                        SzuletesiIdo = DateTime.Now,
                        EszkozAktivalasDatum = item.AktivalasDatum.Value
                    });
                    cnt++;
                }
                return Request.CreateResponse(HttpStatusCode.OK, result.ToDataSourceResult(), Configuration.Formatters.JsonFormatter);
            }

            throw new StatusError(Enums.ManualEnums.CustomHTTPStatusEnum.HibaMiattNincsMegjelenithetoAdat, GRModulResource.KozpontiSzerverNemElerheto);
        }
    }
}
