﻿using System.Web.Mvc;
using Kreta.BusinessLogic.Helpers;
using Kreta.BusinessLogic.Interfaces;
using Kreta.BusinessLogic.Security;
using Kreta.Core;
using Kreta.Core.Iktato.Poszeidon.Factory.Interface;
using Kreta.Enums.ManualEnums;
using Kreta.Resources;
using Kreta.Web.Areas.GRModul.Logic;
using Kreta.Web.Areas.GRModul.Models;
using Kreta.Web.Helpers;
using Kreta.Web.Models.EditorTemplates;
using Kreta.Web.Security;

namespace Kreta.Web.Areas.GRModul.Controllers
{
    [MvcRoleClaimsAuthorize(true)]
    [MvcRolePackageDenyAuthorize(KretaClaimPackages.IsOnlyAlkalmozott.ClaimValue)]
    [MvcRolePackageAuthorize("KretaClaimPackages.GazdasagiUgyintezo.ClaimValue")]
    public class BeszerzesiIgenyController : Controller
    {
        private IIktatoRepositoryFactory IktatoRepositoryFactory { get; }
        private IktatoServiceConfiguration IktatoServiceConfiguration { get; }
        private readonly IJiraHelper JiraHelper;
        public BeszerzesiIgenyController(IJiraHelper jiraHelper, IIktatoRepositoryFactory iktatoRepositoryFactory, IktatoServiceConfiguration iktatoServiceConfiguration)
        {
            IktatoRepositoryFactory = iktatoRepositoryFactory;
            IktatoServiceConfiguration = iktatoServiceConfiguration;
            JiraHelper = jiraHelper;
        }

        public ActionResult Index()
        {
            BeszerzesiIgenySearchModel model = new BeszerzesiIgenySearchModel();

            var helper = new BeszerzesiModulHelper(ConnectionTypeExtensions.GetActiveSessionConnectionType());
            model.IsVezeto = helper.IsLeader(ClaimData.FelhasznaloId);

            return View(model);
        }

        public ActionResult OpenBeszerzesiIgenyDetailPopUp(int id) => GetBeszerzesiIgenyDetailPopUp(id, false);

        public ActionResult OpenBeszerzesiIgenyModifyPopUp(int id) => GetBeszerzesiIgenyDetailPopUp(id, true);

        public ActionResult GetBeszerzesiIgenyDetailPopUp(int id, bool isModosithato)
        {
            var model = new BeszerzesiIgenyLogic(JiraHelper, IktatoRepositoryFactory, IktatoServiceConfiguration).GetBeszerzesiIgenyModel(id);
            model.IsModosithato = isModosithato;

            var pm = new PopUpModel(model, "BeszerzesiIgeny_Bevitel");
            pm = pm.AddCancelBtn(pm, "BeszerzesiIgenyHelper.CancelBeszerzesiIgenyWindow");

            if (isModosithato)
            {
                if (model.IsVezeto)
                {
                    if (model.RogzitoId == ClaimData.FelhasznaloId)
                    {
                        pm = pm.AddBtn(pm, "elfogadasBtn", GRModulResource.Elfogadas, "BeszerzesiIgenyHelper.saveBeszerzesiIgenyElfogadas");
                        pm = pm.AddBtn(pm, "elutasitasBtn", GRModulResource.Torles, "BeszerzesiIgenyHelper.saveBeszerzesiIgenyElutasitas");
                    }
                    else
                    {
                        pm = pm.AddBtn(pm, "elfogadasBtn", GRModulResource.Elfogadas, "BeszerzesiIgenyHelper.saveBeszerzesiIgenyElfogadas");
                        pm = pm.AddBtn(pm, "hpBtn", GRModulResource.HPVisszakuldes, "BeszerzesiIgenyHelper.openBeszerzesiIgenyHPWindow");
                        pm = pm.AddBtn(pm, "elutasitasBtn", GRModulResource.Elutasitas, "BeszerzesiIgenyHelper.saveBeszerzesiIgenyElutasitas");
                    }
                }
                else
                {
                    pm = pm.AddBtn(pm, "modifyBtn", GRModulResource.Modositas, "BeszerzesiIgenyHelper.saveBeszerzesiIgeny", containerCssClass: "BtnOk");
                }
            }

            return PartialView(Constants.General.PopupView, pm);
        }

        public ActionResult OpenBeszerzesiIgenyNewPopUp()
        {
            var model = new BeszerzesiIgenyLogic(JiraHelper, IktatoRepositoryFactory, IktatoServiceConfiguration).GetBeszerzesiIgenyModel(null);
            model.IsModosithato = true;
            model.IgenyAdatokModel.IsPalyazat = (int)IgenNemEnum.Nem;

            PopUpModel pm = new PopUpModel(model, "BeszerzesiIgeny_Bevitel");
            pm = pm.AddCancelBtn(pm, "BeszerzesiIgenyHelper.CancelBeszerzesiIgenyWindow");
            pm = pm.AddBtn(pm, "saveBtn", GRModulResource.Rogzites, "BeszerzesiIgenyHelper.saveBeszerzesiIgeny", containerCssClass: "BtnOk");

            return PartialView(Constants.General.PopupView, pm);
        }
    }
}
