﻿using System;
using System.ComponentModel.DataAnnotations;
using Foolproof;
using Kreta.BusinessLogic.Classes;
using Kreta.BusinessLogic.HelperClasses;
using Kreta.Enums.ManualEnums;
using Kreta.Resources;
using Kreta.Web.Security;

namespace Kreta.Web.Areas.GRModul.Models
{
    public class KotvallModel
    {
        public int? Id { get; set; }
        public bool? IsInfoView { get; set; }

        [Display(Name = nameof(KotvallKezelesResource.BejelentoNeve), ResourceType = typeof(KotvallKezelesResource))]
        public string BejelentoNeve { get; set; }

        [Required(ErrorMessageResourceName = nameof(KotvallKezelesResource.BeszerzesTargyaKitolteseKotelezo), ErrorMessageResourceType = typeof(KotvallKezelesResource))]
        [Display(Name = nameof(KotvallKezelesResource.BeszerzesTargya), ResourceType = typeof(KotvallKezelesResource))]
        public int? BeszerzesTargyId { get; set; }

        [Display(Name = nameof(KotvallKezelesResource.BeszerzesTargya), ResourceType = typeof(KotvallKezelesResource))]
        public string BeszerzesTargyNeve { get; set; }

        [Required(ErrorMessageResourceName = nameof(KotvallKezelesResource.BeszerzesLeirasaKitolteseKotelezo), ErrorMessageResourceType = typeof(KotvallKezelesResource))]
        [Display(Name = nameof(KotvallKezelesResource.BeszerzesLeirasa), ResourceType = typeof(KotvallKezelesResource))]
        public string BeszerzesLeirasa { get; set; }

        [Display(Name = nameof(KotvallKezelesResource.VisMajorIgeny), ResourceType = typeof(KotvallKezelesResource))]
        public bool VisMajorIgeny { get; set; }

        [Display(Name = nameof(KotvallKezelesResource.VisMajorIgeny), ResourceType = typeof(KotvallKezelesResource))]
        public string VisMajorIgeny_Str => VisMajorIgeny ? IgenNemEnum.Igen.GetDisplayName(ClaimData.SelectedTanevID.Value) : IgenNemEnum.Nem.GetDisplayName(ClaimData.SelectedTanevID.Value);

        [RequiredIf("VisMajorIgeny", true, ErrorMessageResourceName = nameof(KotvallKezelesResource.VisMajorIndokKitolteseKotelezo), ErrorMessageResourceType = typeof(KotvallKezelesResource))]
        [Display(Name = nameof(KotvallKezelesResource.VisMajorIndok), ResourceType = typeof(KotvallKezelesResource))]
        public string VisMajorIndok { get; set; }

        [Required(ErrorMessageResourceName = nameof(KotvallKezelesResource.VarhatoBruttoErtekKitolteseKotelezo), ErrorMessageResourceType = typeof(KotvallKezelesResource))]
        [Display(Name = nameof(KotvallKezelesResource.VarhatoBruttoErtek), ResourceType = typeof(KotvallKezelesResource))]
        public int VarhatoBruttoErtek { get; set; }

        public string OktatasiAzonosito { get; set; }
        public Guid IntezmenyEgyediAzonosito { get; set; }
        public string LetrehozoEmail { get; set; }
        public int? TanevId { get; set; }
        public Guid IdpEgyediAzonosito { get; set; }
        public string EloTag { get; set; }
        public string CsaladiNev { get; set; }
        public string Utonev { get; set; }

        public static KotvallItemCo ConvertToCo(KotvallModel model)
        {
            return new KotvallItemCo
            {
                Id = model.Id,
                BejelentoNeve = model.BejelentoNeve,
                TargyId = model.BeszerzesTargyId.Value,
                Leiras = model.BeszerzesLeirasa,
                VisMajorIndok = model.VisMajorIgeny ? model.VisMajorIndok : string.Empty,
                BruttoErtek = model.VarhatoBruttoErtek,
                TanevId = model.TanevId,
                OktatasiAzonosito = model.OktatasiAzonosito,
                LetrehozoEmail = model.LetrehozoEmail,
                BejelentoIdpEgyediAzonosito = model.IdpEgyediAzonosito,
                EloTag = model.EloTag,
                CsaladiNev = model.CsaladiNev,
                Utonev = model.Utonev,
                IntezmenyEgyediAzonosito = model.IntezmenyEgyediAzonosito
            };
        }

        public static KotvallModel ConvertToModel(KotvallItemCo co)
        {
            return new KotvallModel
            {
                BejelentoNeve = co.BejelentoNeve,
                BeszerzesTargyId = co.TargyId,
                BeszerzesTargyNeve = co.BeszerzesTargya,
                BeszerzesLeirasa = co.Leiras,
                VisMajorIndok = co.VisMajorIndok,
                VisMajorIgeny = !string.IsNullOrWhiteSpace(co.VisMajorIndok),
                VarhatoBruttoErtek = co.BruttoErtek,
                TanevId = co.TanevId,
                OktatasiAzonosito = co.OktatasiAzonosito,
                LetrehozoEmail = co.LetrehozoEmail,
                IdpEgyediAzonosito = co.BejelentoIdpEgyediAzonosito,
                EloTag = co.EloTag,
                CsaladiNev = co.CsaladiNev,
                Utonev = co.Utonev,
                IntezmenyEgyediAzonosito = co.IntezmenyEgyediAzonosito
            };
        }
    }
}
